#ifndef Functions_1_h
#define Functions_1_h

#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <algorithm>


/*______________________________CREATE_DATA_______________________________*/

void createData_with_stream() {
    std::string filename = "test_file.txt";
    std::ofstream file_out(filename);
    for (int i = 0; i < 100; ++i){
        file_out << std::setw(3) << " " <<(rand() % 101 - 50) << std::endl;
    }
}

void createData_with_generate() {
    std::string filename = "test_file.txt";
    std::ofstream file_out(filename);
    
    std::vector<int> v(100); //!!! Функции не должны зависеть от типа контейнера
    
    generate(v.begin(), v.end(),[] () {return (rand() % 101 - 50);});
    std::ostream_iterator<int> file_i(file_out, " ");
    std::copy(v.begin(), v.end(), file_i);
}

/*______________________________LOAD_DATA_______________________________*/

template <typename T_ld> //!!! T_ld - не понятное имя
T_ld loadData_with_stream() {
    std::string filename = "test_file.txt";
    std::ifstream file_in(filename);
    if (!file_in) {
        std::cerr << "Error opening file! " << std::endl;
        exit(EXIT_FAILURE);
    }
    T_ld cont;
    using Value_type = typename T_ld::value_type;
    Value_type num;
    while (!file_in.eof()) {
        file_in >> num;
        cont.push_back(num);
    }
    return cont;
}

template <typename T_ld>
T_ld loadData_with_copy() {
    std::string filename = "test_file.txt";
    std::ifstream file_in(filename);
    if (!file_in) {
        std::cerr << "Error opening file! " << std::endl;
        exit(EXIT_FAILURE);
    }
    T_ld cont;
    
    //!!!  Функции не должны зависеть от конкретного типа (int)
    std::copy(std::istream_iterator<int>(file_in), std::istream_iterator<int>(), std::back_inserter(cont));
    
    return cont;
}

/*_______________________________MODIFY_VAR14_____________________________*/

template<typename T_ld>
void Modify_template(T_ld& modify_cont) {
    using Value_type = typename T_ld::value_type;
    Value_type sum = 0;
    Value_type negative_sum = 0;
    for (auto& num : modify_cont) { //!!! Где const у num 
        if (num < 0) {
            negative_sum += num;
        }
    }
    for (auto& num : modify_cont) {
        num += negative_sum/2;
        sum += num;
    }
    modify_cont.push_back(sum);
    modify_cont.push_back(abs(sum / 100));
}

template<typename T_ld>
void Modify_with_iter(const T_ld& it_begin, const T_ld& it_end) {
    double negative_sum = 0; //!!!  Функции не должны зависеть от конкретного типа (int)
    for (auto num = it_begin; num != it_end; ++num) {
            if (*num < 0) {
                negative_sum += *num;
            }
    }
    for (auto num = it_begin; num != it_end; ++num) {
        *num += negative_sum/2;
    }
}

template<typename T_ld>
void Modify_with_for_each(T_ld& modify_cont) {
    using Value_type = typename T_ld::value_type;
    Value_type negative_sum = 0;
    for_each(modify_cont.begin(), modify_cont.end(), [&negative_sum](auto &num) { //!!! Где const 
        if (num < 0) {
            negative_sum += num;
        }
    });
    for_each(modify_cont.begin(), modify_cont.end(), [negative_sum](auto &num) {
        num += negative_sum/2;
    });
}

void Modify_with_transform_and_fstream() {
    std::string filename = "test_file.txt";
    std::ifstream file_in(filename);
    if (!file_in) {
        std::cerr << "Error opening file! " << std::endl;
        exit(EXIT_FAILURE);
    }
    double negative_sum = 0;
    std::string new_filename = "new_test_file.txt";
    std::ofstream file_out(new_filename);
    
     //!!!  Функции не должны зависеть от конкретного типа (int)
    for_each(std::istream_iterator<int>(file_in), std::istream_iterator<int>(), [&negative_sum](auto &num) { //!!! Где const 
        if (num < 0) {
           negative_sum += num;
        }
    });
    
     //!!!  Функции не должны зависеть от конкретного типа (int, double)
     //!!! У меня в экран код не влезает - отформатируйте (он должен быть не больше 100 символов везде)
     //!!! (auto num){return num += negative_sum/2;}) - и какой тут смыcл. Сам контейнер не изменяется
    std::transform(std::istream_iterator<int>(file_in), std::istream_iterator<int>(), std::ostream_iterator<double>(file_out, " "), [&negative_sum](auto num){return num += negative_sum/2;});
}

/*_______________________________OUTPUT_RESULT______________________________*/

template<typename T_ld>
void outputResult_with_fstream(const T_ld& cont) {
    std::string output_filename = "output_test_file.txt";
    std::ofstream file_out(output_filename);
    for (auto &num : cont) { //!!! Где const 
        file_out << num << " ";
    }
}

template<typename T_ld>
void outputResult_with_copy(const T_ld& cont) {
    std::string output_filename = "output_test_file.txt";
    std::ofstream file_out(output_filename);
    using Value_type = typename T_ld::value_type;
    std::copy(cont.begin(), cont.end(), std::ostream_iterator<Value_type>(file_out, " "));
}

#endif
